/**
 * 
 *  JMPI Ping Pong
 *
 */  	
import java.io.*;
import java.text.NumberFormat;
import java.lang.Math;
import mpi.*;

import CCJ.*;
import java.io.Serializable;

class PingPong extends ColMember {

private ColGroup group;
private ColGroupMaster groupMaster;
private static final String groupName = "myGroup";

	static private byte A[];	
	static private int  iterations, size;





    PingPong(String[] args, ColGroupMaster groupMaster) throws CCJException {

	this.groupMaster = groupMaster;

	int numberOfCpus = groupMaster.getNumberOfCpus();

	if (numberOfCpus != 2) {System.err.println("Need exactly 2 cpus.");System.exit(1);}

	groupMaster.addMember(groupName, this);
	group = groupMaster.getGroup("myGroup",numberOfCpus);


  	size = Integer.parseInt(args[0]);
 	iterations = Integer.parseInt(args[1]);	
	
//	System.out.println("Son ----------->>>>>>>>>>"+size+" E "+iterations);
	
	begin();
    }

    public void run() throws CCJException {

	int         i;
	long 		startwtime, endwtime;
    double      time;
    int         ns, my_pe, npes, pot2size;
	
	    my_pe = group.getRank(this);
	    npes  = group.size();

	    barrier(group);

	    A = new byte [size];
		
	    for (i = 0; i < size; i++) {
		    A[i] = (byte)'0'; //(double) 1. / (i + 1);
    	}
	
		pot2size = (int)(Math.log((double) size)/1.386294361);
		if (pot2size < 0) pot2size = 0;  

    if (my_pe == 0) {
	    for (ns = 0; ns < iterations; ns++) {

	    	barrier(group);
			
	        startwtime = System.currentTimeMillis();	
  			for (i = 0; i < 10; i++) {	

			    send_async(group, A, 1 - my_pe);
			    receive(group, 1 - my_pe);				
				
			}  
			endwtime = System.currentTimeMillis();

			//Format the Number to Display
			NumberFormat nf = NumberFormat.getInstance();
            nf.setMaximumFractionDigits(6);
            nf.setMinimumFractionDigits(6);			  
			
			time = (double) ((endwtime - startwtime)/20000.);	// /2 /10 /1000	de ms
			System.out.println(nf.format(time)+" \t "+(long) (1000000*time)+" \t "+nf.format(size/(time*1000000))+" \t "+size+" \t "+pot2size);
		}
	}
	if (my_pe == 1) {
	    for (ns = 0; ns < iterations; ns++) {
		    barrier(group);			
			for (i = 0; i < 10; i++) {
			    receive(group, 1 - my_pe);
				send_async(group, A, 1 - my_pe);		
			}
	    }
	}



	    groupMaster.deleteMember(groupName, this);	
	    System.exit(0);
	}




    public static void main(String[] args) {
	try {
	    ColGroupMaster groupMaster = new ColGroupMaster(args);
	    new PingPong(args, groupMaster);
		
	} catch (CCJException e) {
	    System.err.println("Error in PingPong constructor: " + e);
	    e.printStackTrace();
	    System.exit(1);
	}
    }
}				
		
